#ifdef __USAGE
Wipefile v1.2  - Secure erasure of datafiles.
Copyright (C) 1994 Trever Miller  (bug@cyberdex.cuug.ab.ca)

This program is free software; you can redistribute it and/or modify        
it under the terms of the GNU General Public License as published by        
the Free Software Foundation; either version 2 of the License, or           
(at your option) any later version.                                         

This program is distributed in the hope that it will be useful,             
but WITHOUT ANY WARRANTY; without even the implied warranty of              
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               
GNU General Public License for more details.                                

You should have received a copy of the GNU General Public License           
along with this program; if not, write to the Free Software                 
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

Usage: 
%C [-l | [-p #] [file1 [,file2,...,fileN]] ]

Where:
  file1...fileN    is the list of files you wish to be secure-erased.

  #                is the number of passes (default 1 set of 4 patterns).

  -l               displays the full license agreement.

Example:

%C -p 14 top_secret_file

[Assuming you are excessively paranoid that the default 3 passes isn't enough.]

#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>

static const char RCSID[]=\
"$Id: wipefile.c,v 1.17 1994/01/25 08:29:06 bug Exp bug $";

void wipefile(char *filename, int paranoia_level);

extern void show_gpl(void);

typedef struct wipe_tag {
	char bits[512];
} wipe_t;

wipe_t Pattern[4];

char *Progname;

int main(int argc, char **argv)
{
	extern int optind, 
	           opterr, 
	           optopt;

	int        i,
	           paranoia_level = 1;

	Progname = argv[0];

	printf("\nWipefile v1.2  Copyright (C) 1994 Trever Miller");
	printf(" (bug@cyberdex.cuug.ab.ca)\n\n");
	printf("Wipefile comes with ABSOLUTELY NO WARRANTY; for details ");
	printf("type \"use %s\"\n",Progname);
	printf("This is free software, and you are welcome to redistribute it ");
	printf("under \ncertain conditions (type \"%s -l\").\n\n",Progname);

	while((i = getopt(argc,argv,"l?hp:")) != -1)
	{
		switch(i) 
		{
			case 'l': show_gpl();
			          return(EXIT_SUCCESS);
			          break;

			case 'p': paranoia_level = atoi(optarg);
			          break;

			case '?': 
			case 'h': 
			default : print_usage(argv); 
			          return(EXIT_SUCCESS); 
			          break;
		}
	}

	if(paranoia_level <= 0 || paranoia_level > 42 )
	{
		printf("\nInvalid paranoia_level.  Resetting to 1 set of passes.\n");
		printf("(1010,0101,1111,0000)\n");
		paranoia_level = 1;
	}

	for(i=0;i<sizeof(Pattern[1].bits);i++)
	{
		Pattern[0].bits[i] = 0xAA;  /* 1010 1010 */
		Pattern[1].bits[i] = 0x55;  /* 0101 0101 */
		Pattern[2].bits[i] = 0xFF;  /* 1111 1111 */
		Pattern[3].bits[i] = 0x00;  /* 0000 0000 */
	}


	for(i = optind; i < argc; i++)
	{
		wipefile(argv[i],paranoia_level);
	}

	return(EXIT_SUCCESS);

} /* main() */


void wipefile(char *filename, int paranoia_level)
{

	int         fd,
	            paranoia,
	            bytes,
	            p_count;

	off_t       top_of_file;

	struct stat fileinfo;

	printf("Wiping file %s \n",filename); fflush(stdout);

	if(access(filename,W_OK))
	{
		perror(Progname);
		return;
	}

	if(stat(filename, &fileinfo))
	{
		perror(Progname);
		return;
	}

	if(!S_ISREG(fileinfo.st_mode))
	{
		fprintf(stderr,"%s: %s is not a regular file - skipping.\n",
		        Progname,filename);
		return;
	}

	if((fd = open(filename,O_WRONLY)) == -1)
	{
		perror(Progname);
		return;
	}

	top_of_file = lseek(fd,0L,SEEK_CUR);

	for(paranoia = 1; paranoia <= paranoia_level; paranoia++)
	{
	    for(p_count = 0; p_count < 4; p_count++)
	    {
			switch(p_count)
			{
				case 0: printf("1010 "); break;
				case 1: printf("0101 "); break;
				case 2: printf("1111 "); break;
				case 3: printf("0000 "); break;
				default : printf("Whoa bucky! err in line %s\n",__LINE__);
						  fflush(stdout); exit(EXIT_FAILURE);
			}
			fflush(stdout);

			for(bytes = 0; bytes < fileinfo.st_size; bytes+=sizeof(wipe_t))
			{
				if(write(fd,&Pattern[p_count],sizeof(wipe_t)) != sizeof(wipe_t))
				{
					printf("\n"); fflush(stdout);
					perror(Progname);
					close(fd); 
					exit(EXIT_FAILURE);
				}
			}

			if(fdatasync(fd) == -1)
			{
				printf("\n"); fflush(stdout);
				perror("Error sync'ing file");
				close(fd); 
				exit(EXIT_FAILURE);
			}

			lseek(fd,top_of_file,SEEK_SET);

	    } /* for() p_count */

	} /* for() paranoia...*/

	close(fd);
	remove(filename);
	printf("Wiped.\n\n"); fflush(stdout);

} /* wipefile() */
