#ifdef __USAGE
Wipefree v1.2  - Secure erasure of filesystem freespace.
Copyright (C) 1994 Trever Miller  (bug@cyberdex.cuug.ab.ca)

This program is free software; you can redistribute it and/or modify        
it under the terms of the GNU General Public License as published by        
the Free Software Foundation; either version 2 of the License, or           
(at your option) any later version.                                         

This program is distributed in the hope that it will be useful,             
but WITHOUT ANY WARRANTY; without even the implied warranty of              
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               
GNU General Public License for more details.                                

You should have received a copy of the GNU General Public License           
along with this program; if not, write to the Free Software                 
Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.                   

Usage:
%C [-l | [-p #] [dir1 [,dir2,...,dirN]] ]

Where:
  dir1...dirN    is a list of one directory per filesystem that you
                 wish to have the free space secure-erased.

  #              is the number of passes [default 1 set of 4 patterns].

  -l             displays the full license agreement

ie: given output from "df -h"

File system      (kb) Total     User     Used     Free  Used  Mounted on
//1/dev/hd0t77        50176    50154    20667    29487   41%  /
//1/dev/hd0t78       315392   315302    50333   264969   15%  /partition2

You could secure-erase all the free space on / and /partition2 by the
following command:

%C -l
%C -p 14 /tmp /partition2/tmp

[Assuming that /tmp and /partition2/tmp are writable directories, and
 you're paranoid the default 1 pass of 4 patterns isn't enough.]

#endif

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>
#include <limits.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>

static const char RCSID[]=\
"$Id: wipefree.c,v 1.18 1994/01/25 08:07:48 bug Exp bug $";

extern void show_gpl(void);

void wipefree(char *basedir, int paranoia_level);

typedef struct wipe_tag {
	char bits[512];
} wipe_t;

wipe_t Pattern[4];

char *Progname;

int main(int argc, char **argv)
{
	extern int optind,
	           opterr,
	           optopt;

	int        i,
	           paranoia_level = 1;

	Progname = argv[0];

	printf("\nWipefree v1.2  Copyright (C) 1994 Trever Miller");
	printf(" (bug@cyberdex.cuug.ab.ca)\n\n");
	printf("Wipefree comes with ABSOLUTELY NO WARRANTY; for details ");
	printf("type \"use %s\"\n",Progname);
	printf("This is free software, and you are welcome to redistribute it ");
	printf("under \ncertain conditions (type \"%s -l\").\n\n",Progname);

	while((i = getopt(argc,argv,"l?hp:")) != -1)
	{
		switch(i)
		{
			case 'l': show_gpl();
			          return(EXIT_SUCCESS);
			          break;

		  	case 'p': paranoia_level = atoi(optarg);
		  	          break;

			case '?':
			case 'h':
			default : print_usage(argv);
			          return(EXIT_SUCCESS);
			          break;
		}
	}

	for(i=0;i<sizeof(Pattern[0].bits);i++)
	{
		Pattern[0].bits[i] = 0xAA; /* 1010 1010 */
		Pattern[1].bits[i] = 0x55; /* 0101 0101 */
		Pattern[2].bits[i] = 0xFF; /* 1111 1111 */
		Pattern[3].bits[i] = 0x00; /* 0000 0000 */
	}

	for(i = optind;i < argc; i++)
	{
		wipefree(argv[i],paranoia_level);
	}

	return(EXIT_SUCCESS);

} /* main() */

void wipefree(char *basedir, int paranoia_level)
{
	char  tempfile[PATH_MAX+1] = "";

	int   fd,
	      paranoia,
	      p_count,
	      still_space_left=1;

	off_t top_of_file;


	strcpy(tempfile,basedir);
	strcat(tempfile,"/scrub-bits");

	printf("\nCreating temporary file.\n");
	fflush(stdout);

	if((fd = creat(tempfile,0600)) == -1)
	{
		perror("Error creating tempfile");
		exit(EXIT_FAILURE);
	}

	unlink(tempfile); /* It's still there until we close() it... */

	top_of_file = lseek(fd,0L,SEEK_CUR);

	for(paranoia = 1; paranoia <= paranoia_level; paranoia++)
	{
		for(p_count=0; p_count < 4; p_count++)
		{
			switch(p_count)
			{
				case 0: printf("1010 "); break;
				case 1: printf("0101 "); break;
				case 2: printf("1111 "); break;
				case 3: printf("0000 "); break;
				default: printf("Whoa bucky! err in line %s\n",__LINE__);
				         fflush(stdout); exit(EXIT_FAILURE);
			}
			fflush(stdout);

			while(still_space_left)
			{
				if(write(fd,&Pattern[p_count],sizeof(wipe_t)) != sizeof(wipe_t))
				{
					if(errno != ENOSPC)
					{
						printf("\n"); fflush(stdout);
						perror(Progname);
						close(fd); 
						exit(EXIT_FAILURE);
					}
					else
					{
						still_space_left = 0;
					}
				}
			}

			if(fdatasync(fd) == -1)
			{
				printf("\n"); fflush(stdout);
				perror("Error sync'ing tempfile");
				close(fd); 
				exit(EXIT_FAILURE);
			}

			lseek(fd,top_of_file,SEEK_SET);
			still_space_left = 1;

		} /* for() p_count */

	} /* for() paranoia */

	close(fd);
	printf("Done.\n"); fflush(stdout);

} /* wipefree() */
